@@ -1,6 +1,8 @@
 """Main module."""
 
 import os
+import time
+from datetime import datetime, date, time, timezone
 import json
 
 import shlex
@@ -37,11 +39,32 @@ from aws_pcluster_helpers.models.sinfo import (SInfoTable, SinfoRow)
 
 pcluster_spawner_template_paths = os.path.join(os.path.dirname(__file__), 'templates')
 
+import requests
+import asyncio
+from async_generator import async_generator, yield_
+import pwd
+import os
+import re
+
+import xml.etree.ElementTree as ET
+
+from enum import Enum
+
+from jinja2 import Template
+
+from tornado import gen
+
+from jupyterhub.spawner import Spawner
+from traitlets import Integer, Unicode, Float, Dict, default
+
+from jupyterhub.spawner import set_user_setuid
+
 
 class PClusterSlurmSpawner(batchspawner.SlurmSpawner):
+    # This is tied to the dict returned by the submission_data function
     batch_script = Unicode("""#!/bin/bash
-#SBATCH --output={{homedir}}/jupyterhub_slurmspawner_%j.log
-#SBATCH --job-name=spawner-jupyterhub
+#SBATCH --output={{homedir}}/{{job_prefix}}_%j.log
+#SBATCH --job-name={{job_prefix}}
 #SBATCH --chdir={{homedir}}
 #SBATCH --export={{keepvars}}
 #SBATCH --get-user-env=L
@@ -139,6 +162,8 @@ echo "jupyterhub-singleuser ended gracefully"
 
     _profile_list = None
 
+    job_prefix = Unicode("jupyterhub_spawner")
+
     def _init_profile_list(self, profile_list):
         # generate missing slug fields from display_name
         for profile in profile_list:
@@ -341,6 +366,7 @@ echo "jupyterhub-singleuser ended gracefully"
                 'req_custom_r': '',
                 'req_constraint': first_row.constraint,
                 'exclusive': True,
+                'job_prefix': self.job_prefix,
                 # 'slurm_spawner_table': html_table,
                 'slurm_spawner_table': '',
                 'profile': profile_form,
@@ -349,6 +375,7 @@ echo "jupyterhub-singleuser ended gracefully"
             # TODO If there's nothing in the table we have a problem
             defaults = {
                 'slurm_spawner_table': '',
+                'job_prefix': self.job_prefix,
                 # 'slurm_spawner_table': html_table,
                 'profile': profile_form,
             }
@@ -383,6 +410,9 @@ echo "jupyterhub-singleuser ended gracefully"
         return rtemplate.render(**defaults)
 
     def options_from_form(self, formdata):
+        """
+        The return from this function is what is read into teh SLURM script
+        """
         submission_data = {}
 
         # self.log.debug('--------------------------------')
@@ -431,3 +461,43 @@ echo "jupyterhub-singleuser ended gracefully"
         self.log.debug(submission_data)
 
         return submission_data
+
+    @async_generator
+    async def progress(self):
+        state = self.get_state()
+        dt = datetime.today().strftime('%Y-%m-%d %H:%M:%S')
+        await yield_(
+            {
+                "message": f"""
+                [{dt}]: Job {state['job_id']} submitted. Please watch the progress bar for more information.
+                 You can also run tail -f ~/{self.job_prefix}_{state['job_id']}.log from a terminal.
+                """,
+            }
+        )
+        while True:
+            time.sleep(10)
+            if self.state_ispending():
+                await yield_(
+                    {
+                        "message": f"[{dt}]: Job {state['job_id']} pending in queue...",
+                    }
+                )
+            elif self.state_isrunning():
+                await yield_(
+                    {
+                        "message": f"[{dt}]: Job {state['job_id']} cluster job running... waiting to connect",
+                    }
+                )
+                return
+            else:
+                await yield_(
+                    {
+                        "message": f"[{dt}]: Job {state['job_id']} unknown status...",
+                    }
+                )
+            await gen.sleep(1)
+
+
+def get_ec2_address(address_type='public_ipv4') -> str:
+    response = requests.get(' http://169.254.169.254/latest/meta-data/public-ipv4')
+    return reponse.content.decode('utf-8')
